//  Copyright (c) 2021 NetEase, Inc.  All rights reserved.

#import "NERtcScreenSharePlugin.h"
#import "ScreenShare/NEScreenShareHost.h"
#import "ScreenShare/NEScreenShareHostPrivateDefine.h"

@interface NERtcScreenSharePlugin () <NEScreenShareHostDelegate>

@property(nonatomic, assign) BOOL hasRequestScreenShare;

@property(nonatomic, assign) BOOL isBroadcasting;

@property (nonatomic, strong) NEScreenShareHost * screenShareHost;

@property (nonatomic, strong) FlutterMethodChannel * channel;

@property (nonatomic, assign) NSInteger w_h_rotation;

@property (nonatomic, assign) int receivedFrames;

@property (nonatomic, assign) int pushedFrames;

@property (nonatomic, assign) int reportInterval;

@property(nonatomic, assign) NSTimeInterval lastReportTs;

@end

@implementation NERtcScreenSharePlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    NERtcScreenSharePlugin* instance = [[NERtcScreenSharePlugin alloc] initWithRegistrar:registrar];
}

- (void)detachFromEngineForRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    _channel = nil;
}

- (instancetype)initWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    self = [super init];
    
    self.channel = [FlutterMethodChannel
                methodChannelWithName:@"com.netease.yunxin.flutter.plugin.nertc_screen_share"
                      binaryMessenger:[registrar messenger]];
    [registrar addMethodCallDelegate:self channel:self.channel];
    
    _hasRequestScreenShare = NO;
    _isBroadcasting = NO;
    _w_h_rotation = -1;
    _receivedFrames = 0;
    _pushedFrames = 0;
    _lastReportTs = 0;
    _reportInterval = 10;
    
    return self;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    NSLog(@"IosScreenSharePlugin#handleMethodCall:%@", call.method);
  if ([@"setup" isEqualToString:call.method]) {
    [self setup:call.arguments];
    result(nil);
  }else if ([@"launchBroadcastUI" isEqualToString:call.method]) {
      _hasRequestScreenShare = YES;
      _w_h_rotation = -1;
      _pushedFrames = 0;
      _receivedFrames = 0;
      _lastReportTs = 0;
      BOOL success = NO;
      if (self.isBroadcasting) {
          success = YES;
          NSLog(@"IosScreenSharePlugin#Ignore launchBroadcastUI due to under broadcasting");
      } else if (@available(iOS 12.0, *)) {
          if(_screenShareHost) {
              [_screenShareHost launchBroadcaster];
              success = YES;
          }
      }
      result(@(success));
  } else if ([@"stopBroadcast" isEqualToString:call.method]) {
      _hasRequestScreenShare = NO;
      [self.screenShareHost stopBroadcaster];
      [self onBroadcastStateChanged:NO];
      result(@YES);
  } else if ([@"syncBroadcastState" isEqualToString:call.method]) {
      [self syncBroadcastState];
      result(nil);
  } else {
    result(FlutterMethodNotImplemented);
  }
}

- (void)setup:(NSDictionary*)arguments {
    if (!self.screenShareHost) {
        NEScreenShareHostOptions *options = [[NEScreenShareHostOptions alloc] init];
        options.appGroup = arguments[@"broadcastAppGroup"];
        options.delegate = self;
        _screenShareHost = [[NEScreenShareHost alloc] initWithOptions:options];
        self.reportInterval = [arguments[@"reportInterval"] intValue];
    }
}

#pragma mark - NEScreenShareKitDelegate

- (void)onReceiveVideoFrameInfo:(NSDictionary<NSString *,id> *)videoFrameInfo {
    if (!_hasRequestScreenShare) {
        return;
    }
    [self onBroadcastStateChanged:YES];
    NERtcVideoFrame *frame = [[NERtcVideoFrame alloc] init];
    frame.format = kNERtcVideoFormatI420;
    frame.width = [videoFrameInfo[NEScreenShareVideoFrameWidthKey] unsignedIntValue];
    frame.height = [videoFrameInfo[NEScreenShareVideoFrameHeightKey] unsignedIntValue];
    frame.buffer = (void *)[videoFrameInfo[NEScreenShareVideoFrameDataKey] bytes];
    frame.timestamp = [videoFrameInfo[NEScreenShareVideoFrameTimestampKey] unsignedLongLongValue];
    NSNumber *orientation = videoFrameInfo[NEScreenShareVideoFrameOrientationKey];
    switch (orientation.unsignedIntValue) {
        case kCGImagePropertyOrientationLeft: {
            frame.rotation = kNERtcVideoRotation_90;
            break;
        }
        case kCGImagePropertyOrientationRight: {
            frame.rotation = kNERtcVideoRotation_270;
            break;
        }
        default:
            break;
    }
    if (self.w_h_rotation != frame.width * frame.height * frame.rotation) {
        NSLog(@"IosScreenSharePlugin#onReceiveVideoFrameInfo:width=%@,height=%@,rotation=%@", @(frame.width), @(frame.height), @(frame.rotation));
        self.w_h_rotation = frame.width * frame.height * frame.rotation;
    }
    
    int ret = [NERtcEngine.sharedEngine pushExternalVideoFrame:frame];
    self.receivedFrames++;
    if (ret == 0) {
        self.pushedFrames++;
    }
    NSTimeInterval now = [[NSDate date] timeIntervalSince1970];
    if (self.lastReportTs == 0) {
        self.lastReportTs = now;
    } else if (now - self.lastReportTs > self.reportInterval) {
        //NSLog(@"IosScreenSharePlugin#reportFrameRate=%@", @(_receivedFrames / (now - self.lastReportTs)));
        NSTimeInterval duration = now - self.lastReportTs;
        [self.channel invokeMethod: @"reportBroadcastFps" arguments:@{
            @"receivedFps": @(lround(_receivedFrames / duration)),
            @"pushedFps": @(lround(_pushedFrames / duration))
        }];
        self.receivedFrames = 0;
        self.pushedFrames = 0;
        self.lastReportTs = now;
    }
}

/// 共享开始的回调
- (void)onBroadcastStarted {
    [[NERtcEngine sharedEngine] setExternalVideoSource:YES isScreen:YES];
    [self onBroadcastStateChanged:YES];
}

/// 共享完成的回调
- (void)onBroadcastFinished {
    [self onBroadcastStateChanged:NO];
    [[NERtcEngine sharedEngine] setExternalVideoSource:NO isScreen:YES];
}

- (void)onBroadcastStateChanged:(BOOL)started {
    if (!self.isBroadcasting && started) {
        NSLog(@"IosScreenSharePlugin#onBroadcastStarted");
        self.isBroadcasting = YES;
        [self syncBroadcastState];
    } else if (self.isBroadcasting && !started) {
        NSLog(@"IosScreenSharePlugin#onBroadcastFinished");
        self.isBroadcasting = NO;
        [self syncBroadcastState];
    }
}

- (void)syncBroadcastState {
    if (self.isBroadcasting) {
        [self.channel invokeMethod: @"onBroadcastStarted" arguments:nil];
    } else {
        [self.channel invokeMethod: @"onBroadcastFinished" arguments:nil];
    }
}


@end
