//  Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be found in the LICENSE file.

library nertc_screen_share;

import 'dart:async';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:yunxin_alog/yunxin_alog.dart';


class NERtcScreenShare {
  static const MethodChannel _channel = MethodChannel(
      'com.netease.yunxin.flutter.plugin.nertc_screen_share');

  static const tag = 'NERtcScreenShare';

  /// 共享数据上报时间间隔
  static const reportIntervalInSecond = 10;

  final String? broadcastAppGroup;
  NERtcScreenShareBroadcastCallback? broadcastCallback;
  Completer? _startedCompleter;

  bool _featureSupported = false;

  NERtcScreenShare(this.broadcastAppGroup) {
    _channel.setMethodCallHandler(_handlePlatformMethodCalls);
    _featureSupported = defaultTargetPlatform == TargetPlatform.iOS &&
        broadcastAppGroup != null &&
        broadcastAppGroup!.isNotEmpty;
    if (featureSupported) {
      _channel.invokeMethod('setup', {
        'broadcastAppGroup': broadcastAppGroup,
        'reportInterval': reportIntervalInSecond
      });
      Alog.i(tag: tag, content: 'setup: $broadcastAppGroup');
    }
  }

  Future<dynamic> _handlePlatformMethodCalls(MethodCall call) {
    switch (call.method) {
      case 'onBroadcastStarted':
        {
          Alog.i(tag: tag, content: 'onBroadcastStarted');
          if (_startedCompleter != null) {
            Alog.i(tag: tag, content: 'broadcast started future completes');
            _startedCompleter!.complete();
            _startedCompleter = null;
          }
          broadcastCallback?.onScreenShareBroadcastStarted();
          break;
        }
      case 'onBroadcastFinished':
        {
          Alog.i(tag: tag, content: 'onBroadcastFinished');
          broadcastCallback?.onScreenShareBroadcastFinished();
          break;
        }
      case 'reportBroadcastFps':
        {
          final data = call.arguments as Map?;
          if (data != null) {
            Alog.i(tag: tag,
                content: 'reportBroadcastStatistics: receivedFps=${data['receivedFps']} '
                    'pushedFps=${data['pushedFps']}');
          }
          break;
        }
    }
    return Future.value(null);
  }

  /// 返回true表示打开系统UI成功，否则返回false
  Future<bool> launchBroadcastUI() async {
    Alog.i(tag: tag, content: 'launchBroadcastUI');
    final result = !featureSupported
        ? false
        : await _channel.invokeMethod('launchBroadcastUI') as bool;
    Alog.i(tag: tag, content: 'launchBroadcastUI result=$result');
    return result;
  }

  /// 停止直播进程
  Future<bool> stopBroadcast() async {
    Alog.i(tag: tag, content: 'stopBroadcast');
    final result = !featureSupported
        ? false
        : await _channel.invokeMethod('stopBroadcast') as bool;
    Alog.i(tag: tag, content: 'stopBroadcast result=$result');
    return result;
  }

  bool get featureSupported => _featureSupported;

  Future get broadcastStartedFuture {
    if (_startedCompleter == null) {
      _startedCompleter = Completer();
      //请求plugin同步直播状态
      _channel.invokeMethod('syncBroadcastState');
      Alog.i(tag: tag, content: 'request sync broadcast state');
    }
    return _startedCompleter!.future;
  }

  void setBroadcastCallback(NERtcScreenShareBroadcastCallback broadcastCallback) {
    this.broadcastCallback = broadcastCallback;
  }

  void clearBroadcastCallback() {
    broadcastCallback = null;
    Alog.i(tag: tag, content: 'clearBroadcastCallback');
  }
}

abstract class NERtcScreenShareBroadcastCallback {
  void onScreenShareBroadcastStarted();

  void onScreenShareBroadcastFinished();
}
